;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F676. This file contains the basic code               *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    ZU676.asm                                           *
;    Date:                                                            *
;    File Version:   v1                                                 *
;                                                                     *
;    Author:       NickF                                                   *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F676.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;**********************************************************************

	list      p=16f676           ; list directive to define processor
	#include <p16F676.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.




;***** VARIABLE DEFINITIONS

	CBLOCK 0x20			;      

TenK,Thou,Hund,Tens,Ones,flag
count,t_zn,znk,cyc_tim,bri
NumH,NumL,TEMP1,TEMP0,znak,zn_dac						;NumH,L-HEX , d(n) - DEC 

		ENDC
;flag -    
; 4 - :  - 
; 3 -  - 
;znak - 7-  , znk -  1,2,3,4 
;bri -  
;count,t_zn -  
;cyc_tim -   TMR0
;NumH   ;A3*16+A2
;NumL    ;A1*16+A0
;share variables
;=B4*104 + B3*103 + B2*102 + B1*101 + B0*100
;=B4*10000 + B3*1000 + B2*100 + B1*10 + B0
;TenK     	B4
;Thou     	B3
;Hund     	B2
;Tens     	B1
;Ones		B0

;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    main              ; go to beginning of program

		ORG     0x004             ; interrupt vector location
		goto	obint
		org	0x10
segment		;   DEC ->  .
    CLRF   PCLATH
	ADDWF  PCL, F 
	dt	07E, 00C, 0B6, 09E, 0CC, 0DA, 0FA, 00E, 0FE, 0DE 
;-    -  
;	dt	0FC, 060, 0DA, 0F2, 066, 0B6, 0BE, 0E0, 0FE, 0F6
;-   -    
;*******************************************************************
main
		; init PORTA & SFR

		bcf 	STATUS,RP0 			;Bank 0
		clrf 	PORTA 				;Clear PORTA
		movlw 	07h 				;Set RA<2:0> to 
		movwf 	CMCON 				;comp-off digital I/O
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	ANSEL 				;digital I/O
		clrf 	WPUA 				;  
		movlw 	07h 				;
		movwf 	ANSEL 				;Set PORTA  RA<3>  analog input
		movlw 	0fh
		movwf 	TRISA				;Set PORTA<3..0> as input
		bsf		WPUA,4
		bsf		WPUA,5
		clrf	IOCA				;clear Interrupt-on-Change PORTA
		bsf 	IOCA,3				;    RA3	
		clrf 	PIE1 				;   		
;		movlw 	03h 				;Set POR and BOR 
;		movwf 	PCON 				;  Vdd

		;init PORTC
		bcf 	STATUS,RP0 			;Bank 0
		clrf	PORTC
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	TRISC 				;set PORTC as outputs
		bcf 	STATUS,RP0 			;Bank 0

; these first 4 instructions are not required if the internal oscillator is not use

		call    0x3FF             ; retrieve factory calibration value
		bsf     STATUS,RP0        ; set file register bank to 1 
		movwf   OSCCAL            ; update register with factory cal value 
		bcf     STATUS,RP0        ; set file register bank to 0

; ADC,TMR0
	banksel		ADCON1			
		movlw	030h			;    
		movwf 	ADCON1			;RC  ~ 8 us.
		clrf	TMR0			;  "0"
		movlw	02h				; - 
	banksel		OPTION_REG		;   1/8 
		movwf	OPTION_REG		; CLK  TMR0 - 8 ,   "T0CS"=0
;   256*8=2.048 .
		
; remaining code goes here
start
;  020h-04fh
		movlw	020h			;   
		movwf	FSR
	
c_ind	clrf	INDF			;      FSR
		incf	FSR,f			; FSR
		btfss	FSR,6			;FSR  40h? 
		goto	c_ind			;,     
;  
		movlw	6
		movwf	cyc_tim		;   5
		movwf	bri				; 
		movlw	080h
		movwf	zn_dac
		swapf	zn_dac,f
;	
start_adc
		banksel	PIE1			;     
		bsf		PIE1, ADIE 		;   
		banksel	ADCON0			;   ADCON0
		movlw	4				
		subwf	flag			;  4
		skpnz
		goto	tok				;,   tok
		movlw	081h			;  
		movwf 	ADCON0			;-AN0, Vref = VDD.
		goto	zap_adc
tok		movlw	0C9h			;  
		movwf 	ADCON0			;-AN2, Vref = 1.024V.
zap_adc		bsf		ADCON0,GO		; ADC	
		movlw	6
		movwf	cyc_tim			;   "TMR0'	
; 
		bcf 	STATUS,RP0 		;Bank 0
		bsf		INTCON,PEIE		;   
		bsf		INTCON,RAIE		;    RA3
		bsf		INTCON,T0IE		;    TMR0
		bsf		INTCON,GIE		; 
;   
		call	mult			; /     5
		call	Bin2DecFast		; /  HEX -> DEC

		movlw	1
		movwf	znk			; 1  
st3
		movlw	020h		;   
		addwf	znk,w
		movwf	FSR
		movf	INDF,w		;      
		call	segment		;/    7  
;      
		movwf	znak		;akk->znak
		call	write_zn	;/    

		call	button		;  ,   /  
		clrf	flag
loop
		movf	cyc_tim,f
		skpz
		goto	loop
		incf	znk,1
		movlw	5
		subwf	znk,0
		SKPZ
		goto	st3
;   ,     ,    
		goto	start_adc
;/     LED 
;  /   znak 
;7-      
write_zn
		clrf	t_zn
		movf	znk,w		; "znk"  "w"
		movwf	TEMP0
		btfsc	znk,1		;
		goto	ww			;znk = 3
		btfsc	znk,0		
		bsf		znak,0		;znk = 1,  
ww		bsf		STATUS,C		
shi		rrf		t_zn,f		;     
		decfsz	TEMP0,f		;     
		goto	shi
		movf	t_zn,w
		xorwf 	zn_dac,w	;  "zn_dac"  "t_zn"
;*******************************
;		xorlw	0f0h		; ,     
;*******************************
		movwf	TEMP1		;    "zn_dac"
		movf	znak,w
;*******************************
		xorlw	0ffh		; ,     
;*******************************
		movwf	TEMP0		;    "znak"
		call	shift
		nop
		call	pause
		return
pause
		movlw	.105
		movwf	TEMP0
pa		decf	TEMP0,f		
		skpz
		goto	pa 
		return
;/  
button
		movlw	.6
		movwf	cyc_tim		;    
		movlw	.1			; 
		movwf	bri			;   (  )
		return	
;copy 13-bit number to temporary
;12-bit code
;Posted by Peter Hemsley on September 18, 2004 at 17 40 56 
;Message   ; Bonus 12-Bit version
  ; Leaner and Meaner, 36 instructions
  ; Ideal for displaying A/D values
Bin2DecFast
		movf    NumH,w
        iorlw   0xF0            ;w=H2-16
        movwf   Tens              ;Tens=H2-16
        addwf   Tens,f            ;Tens=H2*2-32
        addwf   Tens,f            ;Tens=H2*3-48
        movwf   Hund              ;Hund=H2-16
        addlw   -D'5'           ;w=H2-21
        addwf   Hund,f            ;Hund=H2*2-37 Done!
        addlw   D'41'           ;w=H2+20
        movwf   Ones              ;Ones=H2+20
        swapf   NumL,w
        iorlw   0xF0            ;w=H1-16
        addwf   Tens,f            ;Tens=H2*3+H1-64
        addwf   Ones,f            ;Ones=H2+H1+4, C=1
        rlf     Ones,f            ;Ones=(H2+H1)*2+9, C=0
        comf    Ones,f            ;Ones=-(H2+H1)*2-10
        rlf     Ones,f            ;Ones=-(H2+H1)*4-20
        movf    NumL,w
        andlw   0x0F            ;w=H0
        addwf   Ones,f            ;Ones=H0-(H2+H1)*4-20 Done!
        rlf     Tens,f            ;C=0, Tens=H2*6+H1*2-128 Done! 
        movlw   D'5'
        movwf   Thou
        movlw   D'10'
  mod0
        addwf   Ones,f            ;D(X)=D(X)mod10
        decf    Tens,f            ;D(X+1)=D(X+1)+D(X)div10
        skpc
        goto    mod0
  mod1
        addwf   Tens,f
        decf    Hund,f
        skpc
        goto    mod1
  mod2
        addwf   Hund,f
        decf    Thou,f
        skpc
        goto    mod2
		movf	t_zn,w
		addwf	Thou,f
        return
;/    "input" / 
mult					
;copy accumulator to temporary
		clrf	t_zn
		movf	NumH,w
		movwf	TEMP1
		movf	NumL,w
		movwf	TEMP0
;/  10- HEX-  5
;     Input: NumL .. NumH, 10 bits
;    Output: NumL .. NumH, 13 bits
; Code size: 16 instructions
; NumH_NumL*4	
		clrc
		rlf		TEMP0,f
		rlf		TEMP1,f
		rlf		TEMP0,f
		rlf		TEMP1,f	
;  NumH_NumL   TEMP1_TEMP0 
		movf	TEMP0,w
		addwf	NumL,f
		movf	TEMP1,w
		skpnc
		incfsz	TEMP1,w
		addwf	NumH,f
		btfss	NumH,4
		goto	nRet
		movlw	4
		movwf	t_zn
		movlw	0x60
		addwf	NumL,f
		skpc
		goto	nRet
		incf	NumH
nRet
		return
; /

;/  16-   16 
;  "TEMP0",     "TEMP1"
shift
		call	shi_8
;    U5
		bsf		PORTC,0
		nop
		nop
		bcf		PORTC,0
		movf	TEMP0,w
		movwf	TEMP1
		call	shi_8
;    U4
		bsf		PORTC,1
		nop
		nop
		bcf		PORTC,1
		nop
		nop
		bcf		PORTC,2		; 
		return
shi_8
		movlw	.8
		movwf	count		;   
per
		clrc	
		rrf		TEMP1,f
		skpnc
		goto	seg1		; =1
		bcf		PORTA,5		; =0
		goto	seg0
seg1	bsf		PORTA,5
seg0	bsf		PORTA,4		; 
		nop
		nop
		bcf		PORTA,4		; 
		DECFSZ	count,f		;  
		goto	per			;    	
		return

;/  
obint
		movwf	05eh
		movf	STATUS,w
		movwf	05fh				;
		bcf		INTCON,GIE			;  
		btfss	PIR1,ADIF			;  
		goto	int_next			;   
int_ADC
		bcf		PIR1,ADIF			;  ADIF
	banksel		ADRESL
		movf	ADRESL,w
		movwf	NumL				; . 
	banksel		ADRESH
		movf	ADRESH,w
		movwf	NumH				; . 
		goto	isr
int_next
		btfss	INTCON,RAIF			;   	
		goto	it					;   ,   
		bcf		INTCON,RAIF
		movf	znk,w					
		movwf	flag			;  
		goto	isr
it		btfss	INTCON,T0IF		;
		goto	int_err			;   
		bcf		INTCON,T0IF		;  T0IF
		decf	cyc_tim,f		;   "TMR0"
		skpz
		goto	it1				;  ,   
		decf	bri				;   
		skpz
		goto	it1				;   
		bsf		PORTC,2			; 		 
it1		bsf		INTCON,T0IE		;   "TMR0"
isr
		bsf		INTCON,GIE		; 
int_err							; 
		movf	05fh, w			; "STATUS"
		movwf	STATUS
		movf	05eh, w			; "W"
		retfie                  ;return from interrupt

;      RC 
			ORG   0x3FF 
           retlw   .128

; initialize eeprom locations
		
		ORG	0x2100
		DE	0x00, 0x01, 0x02, 0x03

		END                 ; directive 'end of program'
